(function (angular) {

    'use strict';

    var app = angular.module('angular-progress-arc', []);

    app.provider('progressArcDefaults', function () {

        var defaults = {
            size: 200,
            strokeWidth: 20,
            stroke: 'black',
            background: null
        };

        this.setDefault = function (name, value) {
            defaults[name] = value;
            return this;
        };

        this.$get = function () {
            return function (attr) {
                angular.forEach(defaults, function (value, key) {
                    if (!attr[key]) {
                        attr[key] = value;
                    }
                });
            };
        };
    });

    app.directive('progressArc', ['progressArcDefaults', function (progressArcDefaults) {
        return {
            restrict: 'E',
            scope: {
                size: '@', // Size of element in pixels.
                strokeWidth: '@', // Width of progress arc stroke.
                stroke: '@', // Color/appearance of stroke.
                counterClockwise: '@', // Boolean value indicating
                complete: '&', // Expression evaluating to float [0.0, 1.0]
                background: '@'  // Color of the background ring. Defaults to null.
            },
            compile: function (element, attr) {

                progressArcDefaults(attr);

                return function (scope, element, attr) {
                    // Firefox has a bug where it doesn't handle rotations and stroke dashes correctly.
                    // https://bugzilla.mozilla.org/show_bug.cgi?id=949661
                    scope.offset = /firefox/i.test(navigator.userAgent) ? -89.9 : -90;
                    var updateRadius = function () {
                        scope.strokeWidthCapped = Math.min(scope.strokeWidth, scope.size / 2 - 1);
                        scope.radius = Math.max((scope.size - scope.strokeWidthCapped) / 2 - 1, 0);
                        scope.circumference = 2 * Math.PI * scope.radius;
                    };
                    scope.$watchCollection('[size, strokeWidth]', updateRadius);
                    updateRadius();
                };
            },
            template:
                '<svg ng-attr-width="{{size}}" ng-attr-height="{{size}}">' +
                '<circle fill="none" ' +
                'ng-if="background" ' +
                'ng-attr-cx="{{size/2}}" ' +
                'ng-attr-cy="{{size/2}}" ' +
                'ng-attr-r="{{radius}}" ' +
                'ng-attr-stroke="{{background}}" ' +
                'ng-attr-stroke-width="{{strokeWidthCapped}}"' +
                '/>' +
                '<circle fill="none" ' +
                'ng-attr-cx="{{size/2}}" ' +
                'ng-attr-cy="{{size/2}}" ' +
                'ng-attr-r="{{radius}}" ' +
                'ng-attr-stroke="{{stroke}}" ' +
                'ng-attr-stroke-width="{{strokeWidthCapped}}"' +
                'ng-attr-stroke-dasharray="{{circumference}}"' +
                'ng-attr-stroke-dashoffset="{{(1 - complete()) * circumference}}"' +
                'ng-attr-transform="rotate({{offset}}, {{size/2}}, {{size/2}})' +
                '{{ (counterClockwise && counterClockwise != \'false\') ? \' translate(0, \' + size + \') scale(1, -1)\' : \'\' }}"' +
                '/>' +
                '</svg>'
        };
    }]);

    app.service('progressArcService', ['$uibModal', '$templateCache', '$log', function ($uibModal, $templateCache, $log) {

        // 模态对象
        var modalInstance;
        // 改变进度事件
        this.changeProgressEvt = function () {
        };
        // 反馈事件注册成功事件
        this.registerSuccess = function () {
        };

        /**
         * 用于改变事件变量的指向
         * @param targetName
         * @param evt
         */
        this.changeEvtDir = function (targetName, evt) {
            this[targetName] = evt;
        };

        $templateCache.put('progressArcModal.html', "<div modal-render=\"{{$isRendered}}\" tabindex=\"-1\" role=\"dialog\" class=\"modal\"\n" +
            "    uib-modal-animation-class=\"fade\"\n" +
            "    modal-in-class=\"in\"\n" +
            "    ng-style=\"{'z-index': 1050 + index*10, display: 'block'}\">\n" +
            "    <div class=\"modal-dialog\" style='width: 240px;margin-top: 10%;' ng-class=\"size ? 'modal-' + size : ''\"><div uib-modal-transclude></div></div>\n" +
            "</div>\n" +
            "");

        /**
         * 打开进度条
         */
        this.openProgress = function (rs) {

            // 保存注册事件
            this.changeEvtDir('registerSuccess', rs);

            modalInstance = $uibModal.open({
                template: '<progress-arc\n' +
                    'size="{{ size }}"\n' +
                    'stroke="{{ stroke }}"\n' +
                    'stroke-width="{{ strokeWidth }}"\n' +
                    'complete="progress"\n' +
                    'background="{{ background }}"\n' +
                    'counter-clockwise="{{ counterClockwise }}"></progress-arc>\n',
                controller: 'progressArcCtrl',
                size: 'md',
                windowTemplateUrl: 'progressArcModal.html'
            });

            modalInstance.result.then(function (selectedItem) {

            }, function () {
                $log.info('Modal dismissed at: ' + new Date());
            });

        }.bind(this);

        /**
         * 关闭进度条
         */
        this.closeProgress = function () {
            modalInstance.close();
        };

        /**
         * 注册进度受反馈事件
         * @param changeP
         */
        this.registerProgress = function (changeP) {
            /**
             * 改变进度值
             * @param val
             */
            var changeProgress = function (val) {
                val <= 1 ? this.changeProgressEvt(val) : $log.error("progressArcService:输入进度值不能大于1");
            }.bind(this);

            // 保存改变进度事件
            this.changeEvtDir('changeProgressEvt', changeP);

            // 进行注册成功回调
            this.registerSuccess(changeProgress);
        }.bind(this);


        return {
            openProgress: this.openProgress,
            closeProgress: this.closeProgress,
            registerProgress: this.registerProgress
        };
    }]);

    app.controller('progressArcCtrl', function ($scope, progressArcService) {
        $scope.size = 240;
        $scope.progress = 0;
        $scope.strokeWidth = 16;
        $scope.stroke = '#3b80e8';
        $scope.background = '#eeeeee';
        $scope.counterClockwise = '';

        /**
         * 接收进度改变回调
         */
        progressArcService.registerProgress(function (val) {
            $scope.progress = val;
        })

    });

})(window.angular);
